@@ -126,10 +126,7 @@ def download_dataset(minify: bool = False) -> None:
 
 
 @app.command()
-def categorize(
-    barcode: str,
-    deepest_only: bool = False,
-) -> None:
+def categorize(barcode: str, deepest_only: bool = False) -> None:
     """Predict product categories based on the neural category classifier.
 
     deepest_only: controls whether the returned predictions should only contain the deepmost
@@ -165,8 +162,7 @@ def import_insights(
         help="Type of the prediction to generate, only needed when --generate-from is used",
     ),
     batch_size: int = typer.Option(
-        128,
-        help="Number of insights that are imported in each atomic SQL transaction",
+        128, help="Number of insights that are imported in each atomic SQL transaction"
     ),
     input_path: Optional[pathlib.Path] = typer.Option(
         None,
@@ -210,8 +206,7 @@ def import_insights(
             # Create a new transaction for every batch
             with db.atomic():
                 batch_imported = importer.import_insights(
-                    prediction_batch,
-                    settings.OFF_SERVER_DOMAIN,
+                    prediction_batch, settings.OFF_SERVER_DOMAIN
                 )
                 logger.info(f"{batch_imported} insights imported in batch")
                 imported += batch_imported
@@ -356,10 +351,8 @@ def run_object_detection_model(
         run_nutrition_table_object_detection,
     )
 
-    kwargs = {}
     if model_name == ObjectDetectionModel.universal_logo_detector:
         func: Callable = run_logo_object_detection
-        kwargs["process_logos"] = False
     elif model_name == ObjectDetectionModel.nutrition_table:
         func = run_nutrition_table_object_detection
     else:
@@ -373,7 +366,7 @@ def run_object_detection_model(
                 JOIN.LEFT_OUTER,
                 on=(
                     (ImagePrediction.image_id == ImageModel.id)
-                    & (ImagePrediction.model_name == model_name)
+                    & (ImagePrediction.model_name == model_name.value)
                 ),
             )
             .where(ImagePrediction.model_name.is_null())
@@ -383,9 +376,6 @@ def run_object_detection_model(
             query = query.limit(limit)
         missing_items = list(query)
 
-    if limit:
-        missing_items = missing_items[:limit]
-
     if typer.confirm(f"{len(missing_items)} jobs are going to be launched, confirm?"):
         for barcode, image_id in tqdm.tqdm(missing_items, desc="image"):
             image_url = generate_image_url(barcode, image_id)
@@ -396,88 +386,148 @@ def run_object_detection_model(
                 barcode=barcode,
                 image_url=image_url,
                 server_domain=settings.OFF_SERVER_DOMAIN,
-                **kwargs,
             )
 
 
 @app.command()
-def init_elasticsearch(
-    load_index: bool = False,
-    load_data: bool = True,
-    to_load: Optional[list[str]] = None,
-) -> None:
+def init_elasticsearch(load_data: bool = True) -> None:
     """
     This command is used for manual insertion of the Elasticsearch data and/or indexes
-    for products and categorties.
-
-    to_load specifies which indexes/data should be loaded - supported values are
-    in robotoff.settings.ElasticsearchIndex.
+    for products.
     """
+    from robotoff.elasticsearch import get_es_client
     from robotoff.elasticsearch.export import ElasticsearchExporter
-    from robotoff.settings import ElasticsearchIndex
-    from robotoff.utils import get_logger
-    from robotoff.utils.es import get_es_client
-
-    logger = get_logger()
+    from robotoff.types import ElasticSearchIndex
 
     es_exporter = ElasticsearchExporter(get_es_client())
-
-    if not to_load:
-        return
-
-    for item in to_load:
-        if item not in ElasticsearchIndex.SUPPORTED_INDICES:
-            logger.error(f"Skipping over unknown Elasticsearch type: '{item}'")
-            continue
-        if load_index:
-            es_exporter.load_index(item, ElasticsearchIndex.SUPPORTED_INDICES[item])
-        if load_data:
-            es_exporter.export_index_data(item)
+    es_exporter.load_all_indices()
+    if load_data:
+        es_exporter.export_index_data(ElasticSearchIndex.product)
 
 
 @app.command()
-def add_logo_to_ann(sleep_time: float = 0.5) -> None:
+def add_logo_to_ann(
+    sleep_time: float = typer.Option(
+        0.0, help="Time to sleep between each query (in s)"
+    )
+) -> None:
+    """Index all missing logos in Elasticsearch ANN index."""
+    import logging
     import time
     from itertools import groupby
 
-    import requests
+    import elasticsearch
     import tqdm
 
     from robotoff.logos import add_logos_to_ann, get_stored_logo_ids
-    from robotoff.models import ImageModel, ImagePrediction, LogoAnnotation, db
+    from robotoff.models import (
+        ImageModel,
+        ImagePrediction,
+        LogoAnnotation,
+        LogoEmbedding,
+        db,
+    )
     from robotoff.utils import get_logger
 
     logger = get_logger()
-    seen = get_stored_logo_ids()
+    logging.getLogger("elastic_transport.transport").setLevel(logging.WARNING)
 
+    seen = get_stored_logo_ids()
+    added = 0
     with db:
-        logos_iter = tqdm.tqdm(
-            LogoAnnotation.select()
+        logger.info("Fetching logo embedding to index...")
+        logo_embedding_iter = tqdm.tqdm(
+            LogoEmbedding.select(LogoEmbedding, LogoAnnotation, ImageModel.id)
+            .join(LogoAnnotation)
             .join(ImagePrediction)
             .join(ImageModel)
-            .where(LogoAnnotation.nearest_neighbors.is_null())
             .order_by(ImageModel.id)
             .iterator()
         )
-        for _, logo_batch in groupby(logos_iter, lambda x: x.image_prediction.image.id):
-            logos = list(logo_batch)
+        for _, logo_embedding_batch in groupby(
+            logo_embedding_iter, lambda x: x.logo.image_prediction.image.id
+        ):
+            logo_embeddings = list(logo_embedding_batch)
 
-            if all(logo.id in seen for logo in logos):
+            to_process = [
+                logo_embedding
+                for logo_embedding in logo_embeddings
+                if logo_embedding.logo_id not in seen
+            ]
+            if not to_process:
                 continue
 
-            image = logos[0].image_prediction.image
-            logger.info(f"Adding logos of image {image.id}")
             try:
-                added = add_logos_to_ann(image, logos)
-            except requests.exceptions.ReadTimeout:
-                logger.warning("Request timed-out during logo addition")
-                continue
+                add_logos_to_ann(to_process)
+            except (
+                elasticsearch.ConnectionError,
+                elasticsearch.ConnectionTimeout,
+            ) as e:
+                logger.info("Request error during logo addition to ANN", exc_info=e)
 
-            logger.info(f"Added: {added}")
+            added += len(to_process)
 
             if sleep_time:
                 time.sleep(sleep_time)
 
+    logger.info(f"{added} embeddings indexed")
+
+
+@app.command()
+def import_embedding(
+    input_dir: Path = typer.Argument(
+        ...,
+        exists=True,
+        file_okay=False,
+        dir_okay=True,
+        help="Directory containing .npz files",
+    )
+) -> None:
+    """Import logo embeddings in DB."""
+    import numpy as np
+    import tqdm
+    from more_itertools import chunked
+
+    from robotoff.models import LogoAnnotation, LogoEmbedding, db
+    from robotoff.utils import get_logger
+
+    logger = get_logger()
+    logger.info(f"Importing logo embeddings from {input_dir}")
+
+    with db:
+        existing_logo_ids = set(
+            x[0] for x in LogoAnnotation.select(LogoAnnotation.id).tuples().iterator()
+        )
+        existing_embedding_logo_ids = set(
+            x[0]
+            for x in LogoEmbedding.select(LogoEmbedding.logo_id).tuples().iterator()
+        )
+
+    imported = 0
+    for npz_path in tqdm.tqdm(input_dir.glob("*.npz"), desc="archive"):
+        archive = np.load(npz_path)
+        logo_ids = archive["logo_id"]
+        embeddings = archive["embedding"]
+        assert embeddings.shape[0] == logo_ids.shape[0]
+        assert embeddings.shape[1] == 512
+        assert embeddings.dtype == np.float32
+
+        with db.connection_context():
+            for batch_indices in chunked(range(embeddings.shape[0]), 1000):
+                with db.atomic():
+                    for i in batch_indices:
+                        logo_id = int(logo_ids[i])
+                        if (
+                            logo_id in existing_logo_ids
+                            and logo_id not in existing_embedding_logo_ids
+                        ):
+                            LogoEmbedding.create(
+                                logo_id=logo_id, embedding=embeddings[i].tobytes()
+                            )
+                            imported += 1
+
+    logger.info(f"{imported} embeddings imported")
+
 
 @app.command()
 def import_logos(
@@ -499,37 +549,10 @@ def import_logos(
 
 
 @app.command()
-def export_logo_annotation(
-    output: pathlib.Path,
-    server_domain: Optional[str] = None,
-    annotated: Optional[bool] = None,
-) -> None:
-    from robotoff.models import ImageModel, ImagePrediction, LogoAnnotation, db
-    from robotoff.utils import dump_jsonl
-
-    with db:
-        where_clauses = []
-
-        if server_domain is not None:
-            where_clauses.append(ImageModel.server_domain == server_domain)
-
-        if annotated is not None:
-            where_clauses.append(LogoAnnotation.annotation_value.is_null(not annotated))
-
-        query = LogoAnnotation.select().join(ImagePrediction).join(ImageModel)
-        if where_clauses:
-            query = query.where(*where_clauses)
-
-        logo_iter = query.iterator()
-        dict_iter = (logo.to_dict() for logo in logo_iter)
-        dump_jsonl(output, dict_iter)
-
-
-@app.command()
-def export_logos_ann(
+def export_logos(
     output: pathlib.Path = typer.Argument(
         ...,
-        help="Path to the output file, can either have .json or .json.gz as "
+        help="Path to the output file, can either have .jsonl or .jsonl.gz as "
         "extension",
     ),
 ) -> None:
@@ -542,8 +565,14 @@ def export_logos_ann(
         query = (
             LogoAnnotation.select(
                 LogoAnnotation.id,
+                LogoAnnotation.index,
                 LogoAnnotation.bounding_box,
                 LogoAnnotation.score,
+                LogoAnnotation.annotation_type,
+                LogoAnnotation.annotation_value,
+                LogoAnnotation.annotation_value_tag,
+                LogoAnnotation.taxonomy_value,
+                LogoAnnotation.username,
                 ImageModel.image_id,
                 ImageModel.barcode,
             )
